#
# Copyright (c) 1993-1994 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Computer Systems
#	Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/vint/nam-1/nam.tcl,v 1.22 1997/11/04 19:50:11 haoboy Exp $ (LBL)
#
set tk_strictMotif 0

set uscale(m) 1e-3
set uscale(u) 1e-6
set uscale(k) 1e3
set uscale(K) 1e3
set uscale(M) 1e6

# initialize layout constant
AutoNetworkModel set RANDOM_SEED_ 1
AutoNetworkModel set KCa_ 0.15
AutoNetworkModel set KCr_ 0.15
AutoNetworkModel set AUTO_ITERATIONS_ 10
AutoNetworkModel set INCR_ITERATIONS_ 10

# network model used. can be customized in .nam.tcl in current directory
set NETWORK_MODEL NetworkModel

proc time2real v {
	global uscale
	foreach u [array names uscale] {
		set k [string first $u $v]
		if { $k >= 0 } {
			set scale $uscale($u)
			break
		}
	}
	if { $k > 0 } {
		set v [string range $v 0 [expr $k - 1]]
		set v [expr $scale * $v]
	}
	return $v
}
#XXX
proc bw2real v {
	return [time2real $v]
}

proc yesno attr {
	set v [resource $attr]
	if { [string match \[0-9\]* $v] } {
		return $v
	}
	if { $v == "true" || $v == "True" || $v == "t" || $v == "y" ||
		$v == "yes" } {
		return 1
	}
	return 0
}

proc resource s {
	return [option get . $s Nam]
}

#XXX
proc mapf s { return $s }

option add Nam.foundry adobe startupFile
set ff [option get . foundry Nam]
set helv10 [mapf "-$ff-helvetica-medium-r-normal--*-100-75-75-*-*-*-*"]
set helv10b [mapf "-$ff-helvetica-bold-r-normal--*-100-75-75-*-*-*-*"]
set helv10o [mapf "-$ff-helvetica-bold-o-normal--*-100-75-75-*-*-*-*"]
set helv12 [mapf "-$ff-helvetica-medium-r-normal--*-120-75-75-*-*-*-*"]
set helv12b [mapf "-$ff-helvetica-bold-r-normal--*-120-75-75-*-*-*-*"]
set helv14 [mapf "-$ff-helvetica-medium-r-normal--*-140-75-75-*-*-*-*"]
set helv14b [mapf "-$ff-helvetica-bold-r-normal--*-140-75-75-*-*-*-*"]
set times14 [mapf  "-$ff-times-medium-r-normal--*-140-75-75-*-*-*-*"]

option add *background gray80 startupFile
option add *activeBackground gray90 startupFile
option add *viewBackground gray90 startupFile
option add *font $helv12b startupFile
option add *Font $helv12b startupFile

option add Nam.disablefont $helv10o startupFile
option add Nam.smallfont $helv10b startupFile
option add Nam.medfont $helv12b  startupFile
option add Nam.helpFont $times14 startupFile
option add Nam.entryFont $helv10 startupFile

option add *Radiobutton.relief flat startupFile

option add Nam.rate		2ms	startupFile
option add Nam.movie		0	startupFile
option add Nam.granularity	40	startupFile
option add Nam.pause		1	startupFile

#
# use 2 pixels of padding by default
#
option add *padX 2 startupFile
option add *padY 2 startupFile
#
# don't put tearoffs in pull-down menus
#
option add *tearOff 0 startupFile

proc smallfont { } {
	return [option get . smallfont Nam]
}
proc mediumfont { } {
	return [option get . medfont Nam]
}

proc toggle_window w {
	if ![winfo exists $w] { build$w }
	global created$w
	if ![info exists created$w] {
		set created$w 1
		wm transient $w .
		update idletasks
		set x [winfo rootx .]
		set y [winfo rooty .]
		incr y [winfo height .]
		incr y -[winfo reqheight $w]
		incr y -20
 		# adjust for virtual desktops
		incr x [winfo vrootx .]
		incr y [winfo vrooty .]
		if { $y < 0 } { set y 0 }
		if { $x < 0 } {
			set x 0
		} else {
			set right [expr [winfo screenwidth .] - \
					   [winfo reqwidth $w]]
			if { $x > $right } {
				set x $right
			}
		}
		wm geometry $w +$x+$y
		wm deiconify $w
	} elseif [winfo ismapped $w] {
		wm withdraw $w
	} else {
		wm deiconify $w
	}
}

proc backFrame { } {
	global now timeStep
	settime [expr $now - $timeStep]
}

proc nextFrame { } {
    global now timeStep direction
    if {$direction>0} {
	settime [expr $now + $timeStep]
    } else {
	settime [expr $now - $timeStep]
    }
}

proc net_settime t {}

#
# Set time slider to a tick value between 0 and 1000.
#
set sliderPressed 0
proc settime t {
	#XXX
	net_settime $t    

        global sliderPressed range mintime timeSlider trace now nowDisp \
		maxtime graphName graphs netViews
        if { $t > $maxtime } {
		stop 1
		return
	} elseif { $t < $mintime } {
	        set t $mintime
	}
	set now $t
	set nowDisp [format %.6f $now]
	if { $sliderPressed == 0 } {
	    timesliderset [expr int(1000. * ($now - $mintime) / $range)]
#	    $timeSlider set [expr int(100. * ($now - $mintime) / $range)]
	}
	set event [$trace settime $now $sliderPressed]
	if { [string length $graphName] > 0 } {
		if { [string length $event] > 0 } {
			graph_update $event
		}
	}
	foreach graph $graphs {
	    $graph settime $t
	}
	foreach netView $netViews {
	    update_monitors $netView $now
	}
}

proc draw_data_pnt { tim id } {
	global prevAckId prevPktId clearDataCmd clearAckCmd lastDrawCmd \
	       graphName
	if { $prevAckId != 0 } {
		tkgraph_cmd 0 $clearAckCmd
		set prevAckId 0
	} elseif { $prevPktId != 0 } {
		tkgraph_cmd 0 $clearDataCmd
	}
	set lastDrawCmd "draw_point $graphName $tim $id"
	tkgraph_cmd 1 $lastDrawCmd
	set clearDataCmd "clear_point $graphName $tim $id"
	set prevPktId $id
}

proc draw_ack_pnt { tim id } {
	global prevAckId prevPktId delay01 graphName lastDrawCmd clearAckCmd \
	       clearDataCmd
	if { $prevAckId != 0 } {
		tkgraph_cmd 0 $clearAckCmd
	} elseif { $prevPktId != 0 } {
		tkgraph_cmd 0 $clearDataCmd
		set prevPktId 0
	}
	set prevAckId $id
	set arriv [expr $tim + $delay01]
	set lastDrawCmd "draw_point $graphName $arriv $id"
	tkgraph_cmd 1 $lastDrawCmd
	set clearAckCmd "clear_point $graphName $arriv $id"
}

proc graph_update_interval tim {
	global intervalStart intervalEnd mintime maxtime lastDrawCmd \
	       interval graphName

	# Update graph interval as needed.
	set overlap [expr 0.2 * $interval]
	if { $tim > $intervalStart && $tim < $intervalEnd } {
		return
	}
	if { $tim > $maxtime } {
		stop 1
		return
	}
	if { $tim >= [expr $intervalEnd - $overlap] } {
		set intervalStart [expr $intervalEnd - $overlap]
	} elseif { $tim < $intervalStart } {
		set intervalStart [expr $tim - 0.5 * $interval]
	}
	set intervalEnd [expr $intervalStart + $interval]
		
	# Check if going beyond max x or min x and update interval
	# accordingly.
	if { $intervalEnd > $maxtime } {
		set intervalEnd [expr $maxtime + $overlap]
		set intervalStart [expr $intervalEnd - $interval]
	} elseif { $intervalStart <= $mintime } {
		set intervalStart 0.0
		set intervalEnd [expr $intervalStart + $interval]
	}

	set cmd [format "update_graph %s %.17g %.17g" \
			 $graphName $intervalStart $intervalEnd]
	tkgraph_cmd 1 $cmd
	
	# Redraw last point drawn.
	tkgraph_cmd 1 $lastDrawCmd
}

proc graph_update events {

	set el [split $events /]
	
	foreach e $el {
		scan $e "%d %d %g %g" src dst tim id

		# Draw and/or clear any points as needed.
		if { $src == 0 } {
			# data packet leaving node 0
			draw_data_pnt $tim $id

		} elseif { $dst == 0 } {
			# Ack packet leaving node 1 (to 0).  Just need to save
			# ack id. Point for ack will be drawn later when the
			# first data packet sent as a result of this ack leaves
			# node 0.
			draw_ack_pnt $tim $id
		}
		graph_update_interval $tim
	}
}

proc slidetime { tick remote } {
	global now range mintime trace
	set now [expr ($tick * $range) / 1000. + $mintime]
	settime $now
	if { $remote } {
		peer_cmd 1 "slidetime $tick 0"
	}
}

proc bumpstepper { amt remote } {
	set v [.right.rate get]
	incr v [expr - $amt]
	if { $v > 100 } {
		.right.rate set 100
	} {
		if { $v < 0 } {
			.right.rate set 0
		} {
			.right.rate set $v
		}
	}
	if { $remote } {
		peer_cmd 1 "bumpstepper $amt 0"
	}
}

proc renderFrame { } {
	global running direction sliderPressed granularity
	if { $running && !$sliderPressed } {
		nextFrame
		update idletasks
		after $granularity renderFrame
	}
}

proc redrawFrame { } {
    global now
    settime $now
}

proc remote_play t {
        global timeSlider
        settime $t
        play 0
}

proc play remote {
	global running now
	set running 1
	after 0 renderFrame
	if { $remote } {
		peer_cmd 1 "remote_play $now"
	}
}

proc remote_stop t {
        stop 0
        settime $t
}

proc stop remote {
	global running now
	set running 0
	if { $remote } {
		peer_cmd 1 "remote_stop $now"
	}
}

proc remote_set_time t {
        global timeSlider
        settime $t
}

proc reset { } {
	settime 0.
	peer_cmd 1 "remote_set_time 0."
}

proc rewind { } {
	global now timeStep
	set t [expr $now - $timeStep*25.0]
	settime $t
	peer_cmd 1 "remote_set_time $t"
#	settime 0.
#	peer_cmd 1 "remote_set_time 0."
}

proc fast_fwd { } {
	global now timeStep
        set t [expr $now + $timeStep*25.0]
        settime $t
	peer_cmd 1 "remote_set_time $t"
}

proc next_event { } {
	global trace running
	set t [$trace nxtevent]
	settime $t
	peer_cmd 1 "remote_set_time $t"
	if { !$running } {
		nextFrame
		peer_cmd 1 nextFrame
	}
}

proc step_format t {
	if { $t < 1e-3 } {
		return [format "%.1f" [expr $t * 1e6]]us
	} elseif { $t < 1. } {
		return [format "%.1f" [expr $t * 1e3]]ms
	}
	return [format "%.1f" $t]s
}

proc set_rate { v remote } {
	global timeStep stepDisp rateSlider currRate
	set timeStep [expr pow(10, $v / 10.)]
	set stepDisp [step_format $timeStep]
        if { [$rateSlider get] != $v } { $rateSlider set $v }
	set currRate $v
	if { $remote } {
		peer_cmd 1 "set_rate $v 0"
	}
}

# Set time to its previous value (before it was changed by
# pressing mouse button 1 on the time slider).
proc time_undo { } {
        global timeSlider prevTime now
        set currTime $now
        settime $prevTime
	peer_cmd 1 "settime $prevTime"
        set prevTime $currTime
}

# Set rate to its previous value (before it was changed by
# pressing mouse button 1 on the rate slider).
proc rate_undo { } {
        global prevRate rateSlider
        set tmpRate [$rateSlider get]
        set_rate $prevRate 1
        $rateSlider set $prevRate
        set prevRate $tmpRate
}

proc button_release_1 t {
        global timeSlider
	slidetime $t 1
        $timeSlider set $t
	global sliderPressed
	set sliderPressed 0
}

proc button_press_1 s {
	global sliderPressed prevTime
	set sliderPressed 1
        set prevTime $s
}

proc build.menus w {
        global NETWORK_MODEL showpanel nam_name windows
	frame $w.menu -relief groove -bd 2
	pack $w.menu -side top -fill x

	set padx 4

	set mb $w.menu.file
	set m $mb.m
	menubutton $mb -text "File" -menu $m -underline 0 \
		-borderwidth 1 
	menu $m
	$m add command -label "Open..." -state disabled
	$m add separator
	$m add command -label "Quit" -command done
	pack $mb -side left -padx $padx

	set mb $w.menu.views
	set m $mb.m
	menubutton $mb -text "Views" -menu $m -underline 0 \
		-borderwidth 1 
	menu $m
	$m add command -label "New view" -command new_view
	$m add separator
	$m add checkbutton -label "Show monitors" \
		-variable showpanel(monitor)
        if {$NETWORK_MODEL=="AutoNetworkModel"} {
	    $m add checkbutton -label "Show autolayout" \
		    -variable showpanel(autolayout)
	} else {
	    $m add checkbutton -label "Show autolayout" -state disabled
	}
	$m add checkbutton -label "Show annotations" \
		-variable showpanel(annotate)
	pack $mb -side left -padx $padx

	label $w.menu.name -text $nam_name -font [smallfont] \
		-width 30 -borderwidth 2 -relief groove 
        set windows(title) $w.menu.name
	pack $w.menu.name -side left -fill x -expand true -padx 4 -pady 1

	set mb $w.menu.help
	set m $mb.m
	menubutton $mb -text "Help" -menu $m -underline 0 \
		-borderwidth 1 
	menu $m
	$m add command -label "Help" -command {new_web help}
	$m add command -label "About nam" -command {new_web about}
	pack $mb -side right -padx $padx
}

proc build.v0 {netView w} {
    global magnification viewOffset
    set magnification 1.0
    set viewOffset(x) 0.0
    set viewOffset(y) 0.0
    frame $w.f
    pack $w.f -side top
    button $w.f.b1 -bitmap "zoomin" -command "$netView zoom 1.6" \
	    -highlightthickness 0 -borderwidth 1
    pack $w.f.b1 -side top -ipady 3
    button $w.f.b2 -bitmap "zoomout" -command "$netView zoom 0.625" \
	    -highlightthickness 0 -borderwidth 1
    pack $w.f.b2 -side top -ipady 3
}

proc build.p0b w {
    global timeslider
    frame $w.f -borderwidth 2 -relief groove
    pack $w.f -side top -fill x -expand 1
    set timeslider(height) 12
    set timeslider(swidth) 32
    set timeslider(width) 32
    set timeslider(canvas) $w.f.c
    set timeslider(frame) $w
    canvas $w.f.c -height $timeslider(height) -width 300 \
	    -background white -highlightthickness 0
    pack $w.f.c -side left -fill x -expand 1 -padx 0 -pady 0

    label $w.f.c.b -bitmap time -highlightthickness 0 -borderwidth 1 \
	    -relief raised
    set timeslider(tag) [$w.f.c create window \
	    [expr $timeslider(swidth)/2] 0 -window $w.f.c.b \
	    -height 12 -width $timeslider(swidth) -anchor n]
    set timeslider(pos) 0
    bind $w.f.c <ButtonPress-1> "timeslidertrough $w.f.c %x %y"
    bind $w.f.c.b <ButtonPress-1> "timesliderpress $w.f.c.b %x %y;break"
    bind $w.f.c.b <ButtonRelease-1> "timesliderrelease $w.f.c.b %x %y"
    bind $w.f.c.b <B1-Motion> "timeslidermotion $w.f.c.b %x %y"
    bind $w.f.c <Configure> timeticks
}

proc timeticks {} {
    global timeslider mintime range
    set width [winfo width $timeslider(canvas)]
    if {$width == $timeslider(width)} { return }
    set timeslider(width) $width
    $timeslider(canvas) delete ticks

    #we really shouldn't need to do this but there's a redraw bug in the
    #tk canvas that we need to work around (at least in tk4.2) - mjh
    pack forget $timeslider(canvas)
    update
    update idletasks
    pack $timeslider(canvas) -side left -fill x -expand 1 -padx 0 -pady 0

    set x [expr $timeslider(swidth)/2]
    set intertick [expr ($width-$timeslider(swidth))/(10 * $range)]
    for {set t $mintime} {$t < ($range+$mintime)} {set t [expr $t+0.1]} {
	set intx [expr int($x)]
	$timeslider(canvas) addtag ticks withtag \
		[$timeslider(canvas) create line \
		$intx [expr $timeslider(height)/2] $intx $timeslider(height)]
	set x [expr $x+$intertick]
    }
    set x [expr $timeslider(swidth)/2]
    set intertick [expr ($width-$timeslider(swidth))/($range)]
    for {set t $mintime} {$t < ($range+$mintime)} {set t [expr $t+1]} {
	set intx [expr int($x)]
	$timeslider(canvas) addtag ticks withtag \
		[$timeslider(canvas) create line \
		$intx 0 $intx $timeslider(height)]
	set x [expr $x+$intertick]
    }
}

proc timesliderset {t} {
    global timeslider
#    set timeslider(width) [winfo width $timeslider(canvas)]
    set x [expr ($timeslider(width)-$timeslider(swidth))*$t/1000.0]
    $timeslider(canvas) coords $timeslider(tag) [expr $x + $timeslider(swidth)/2] 0
    set timeslider(pos) $x
}

proc timesliderpress {w x y} {
    global timeslider sliderPressed 
    set timeslider(oldpos) $x
#    set timeslider(width) [winfo width $timeslider(canvas)]
    $w configure -relief sunken
    set sliderPressed 1
}

proc timeslidertrough {w x y} {
    global timeslider sliderPressed 
    if {$timeslider(pos)>$x} {
	rewind
    } else {
	fast_fwd
    }
}

proc timeslidermotion {w x y} {
    global timeslider mintime range nowDisp
    set diff [expr $x - $timeslider(oldpos)]
    $timeslider(canvas) move $timeslider(tag) \
	    $diff 0
    set timeslider(pos) [expr $timeslider(pos) + $diff]
    if {$timeslider(pos)<0} {
	$timeslider(canvas) move $timeslider(tag) \
		[expr 0 - $timeslider(pos)] 0
	set timeslider(pos) 0
    }
    if {$timeslider(pos)>[expr $timeslider(width)-$timeslider(swidth)]} {
	$timeslider(canvas) move $timeslider(tag) \
		[expr ($timeslider(width)-$timeslider(swidth))-$timeslider(pos)] 0
	set timeslider(pos) [expr $timeslider(width)-$timeslider(swidth)]
    }
    set tick 0
    catch {
	set tick [expr 1000.0*$timeslider(pos)/($timeslider(width)-$timeslider(swidth))]
    }
    set now [expr ($tick * $range) / 1000. + $mintime]
    set nowDisp [format %.6f $now]
    set timeslider(tick) $tick
}

proc timesliderrelease {w x y} {
    global timeslider sliderPressed running
    timeslidermotion $w $x $y
    $w configure -relief raised
    slidetime $timeslider(tick) 1
    set sliderPressed 0
    if $running {
	renderFrame
    }
}

proc build.p0 w {
	scale $w.slider -orient horizontal -width 7p \
			-from 0 -to 100 -showvalue false -relief groove \
			-borderwidth 1  -highlightthickness 0 \
			-troughcolor [option get . background Nam]

	#
	# We want slightly different semantics.  Instead of tracking
	# the time slider continuously, we just update it when the
	# button is released.
	# E.g., it takes too long to do a fast-forward each time.
	#
	global timeSlider 
	set timeSlider $w.slider
	bind $timeSlider <ButtonRelease-1> {
	        set t [%W get]
		slidetime $t 1
		global sliderPressed running
		set sliderPressed 0
		peer_cmd 1 "button_release_1 $t"
		if $running {
			renderFrame
		}
	}
	bind $timeSlider <ButtonPress-1> {
		global sliderPressed prevTime
		set sliderPressed 1
	        set prevTime $now
		peer_cmd 1 "button_press_1 $prevTime"
	}
	bind $timeSlider <B1-Motion> {
		global range mintime trace nowDisp
		set tick [%W get]
		set now [expr ($tick * $range) / 100. + $mintime]
		set nowDisp [format %%.6f $now]
	}

#	button $w.rew -bitmap rewind \
#		-command rewind -anchor center -relief ridge
#	button $w.stop -bitmap stop \
#		-command stop -anchor center -relief ridge
#	button $w.idle -bitmap play \
#			-command play -anchor center -relief ridge
#	button $w.ff -bitmap "ff" \
#			-command fast_fwd -anchor center -relief ridge

	pack $w.slider -side left -fill x -expand 1
#	pack $w.rew $w.stop $w.idle $w.ff -side left
}

proc build.p1 {w netView} {
        global rateSlider granularity timeStep currRate prevRate stepDisp

	set f [smallfont]
	frame $w.bar -relief groove -borderwidth 2
        frame $w.bar.rate -borderwidth 1 -relief sunken
	scale $w.bar.rate.s -orient horizontal -width 7p \
		        -label "Step:" -font [smallfont]\
			-from -60 -to -1 -showvalue false \
			-relief flat\
			-borderwidth 1 -highlightthickness 0 \
			-troughcolor [option get . background Nam]
        pack $w.bar.rate.s -side top -fill both -expand true
        set rateSlider $w.bar.rate.s
	set granularity [option get . granularity Nam]
	set timeStep [time2real [option get . rate Nam]]
        set stepDisp [step_format $timeStep]
        set currRate [expr 10*log10($timeStep)]
        set prevRate $currRate
	$rateSlider set $currRate
        bind $rateSlider <ButtonRelease-1> { 
		set v [%W get]
		set_rate $v 1
	}
	bind $rateSlider <ButtonPress-1> {
		global currRate prevRate
		set prevRate $currRate
	}
	bind $rateSlider <B1-Motion> {
		global timeStep stepDisp
		set v [%W get]
		set timeStep [expr pow(10, $v / 10.)]
		set stepDisp [step_format $timeStep]
	}
	trace variable stepDisp w {displayStep $stepDisp}


#	label $w.bar.timer -text Time: -font $f -borderwidth 1
	label $w.bar.timerVal -textvariable nowDisp -width 10 -anchor w -font $f \
		-borderwidth 1 -relief sunken -anchor e
#	label $v.step -text "  Step:" -font $f -borderwidth 1
#	label $w.bar.stepVal -textvariable stepDisp -width 8 -anchor w -font $f \
#		-borderwidth 2 -relief groove -anchor e


#	button $w.help -text Help -borderwidth 2 -relief raised \
#		-font $f -command "toggle_window .help" -width 5

        frame $w.bar.run
	button $w.bar.run.b -bitmap play -borderwidth 1 -relief raised \
		-highlightthickness 0 -font $f  \
		-command "focus $netView;\
			  set direction 1;\
		          if {(\$running!=1)} {set running 1;renderFrame};\
                          highlight $w.bar.run 1"
        frame $w.bar.run.f -height 5 -relief sunken \
		-borderwidth 1
        pack $w.bar.run.b -side top -fill both -expand true
        pack $w.bar.run.f -side top -fill x

        frame $w.bar.back
	button $w.bar.back.b -bitmap back -borderwidth 1 -relief raised \
		-highlightthickness 0 -font $f \
		-command "set direction -1;\
		          if {(\$running!=1)} {set running 1;renderFrame};\
			  highlight $w.bar.back 1"
        frame $w.bar.back.f -height 5 -relief sunken \
		-borderwidth 1
        pack $w.bar.back.b -side top -fill both -expand true
        pack $w.bar.back.f -side top -fill x
        
        frame $w.bar.stop
	button $w.bar.stop.b -bitmap stop -borderwidth 1 -relief raised \
		-highlightthickness 0 -font $f \
		-command "set running 0;\
		          renderFrame;\
			  highlight $w.bar.stop 1"
        frame $w.bar.stop.f -height 5 -relief sunken \
		-borderwidth 1
        pack $w.bar.stop.b -side top -fill both -expand true
        pack $w.bar.stop.f -side top -fill x

        frame $w.bar.rew
	button $w.bar.rew.b -bitmap rew -borderwidth 1 -relief raised \
		-highlightthickness 0 -font $f -command rewind
        frame $w.bar.rew.f -height 5 -relief sunken \
                -borderwidth 1
        pack $w.bar.rew.b -side top -fill both -expand true
        pack $w.bar.rew.f -side top -fill x

        frame $w.bar.ff
	button $w.bar.ff.b -bitmap ff -borderwidth 1 -relief raised \
		-highlightthickness 0 -font $f -command fast_fwd
        frame $w.bar.ff.f -height 5 -relief sunken \
                -borderwidth 1
        pack $w.bar.ff.b -side top -fill both -expand true
        pack $w.bar.ff.f -side top -fill x


	button $w.bar.quit -bitmap eject -borderwidth 1 -relief raised \
		-highlightthickness 0 -font $f -command done

	pack $w.bar.rate -side right -fill y
	pack $w.bar.timerVal -side right -pady 0 \
		-ipady 1 -padx 1 -fill y
        pack $w.bar.quit -side right -padx 1 -pady 1 -fill both -expand true
        pack $w.bar.ff -side right -padx 1 -pady 1 -fill both -expand true
        pack $w.bar.run -side right -padx 1 -pady 1 -fill both -expand true
        pack $w.bar.stop -side right -padx 1 -pady 1 -fill both -expand true
        pack $w.bar.back -side right -padx 1 -pady 1 -fill both -expand true
        pack $w.bar.rew -side right -padx 1 -pady 1 -fill both -expand true
#        pack $w.help -side left -padx 1 -pady 1 -fill y 

	pack $w.bar -fill x -expand 1 -side right
        global prevbutton
        set prevbutton $w.bar.stop

	# start out stopped
	highlight $w.bar.stop 1
}

proc displayStep {stepDisp args} {
    global rateSlider
    $rateSlider configure -label "Step: $stepDisp"
}

proc build.p2 {w} {
	global ITERATIONS KCa KCr NETWORK_MODEL
	set f [smallfont]
#	frame $w.bar -relief ridge -borderwidth 2
        label $w.label -text "Auto layout:" -font $f
	label $w.label_ca -text "Ca" -font $f
	entry $w.inputca -width 6 -relief sunken -textvariable KCa -font $f
	label $w.label_cr -text "Cr" -font $f
	entry $w.inputcr -width 6 -relief sunken -textvariable KCr -font $f
	label $w.label_iter -text "Iterations" -font $f
	entry $w.inputiter -width 6 -relief sunken -textvariable ITERATIONS

	button $w.relayout -text re-layout -relief raised -font $f \
		-command "nam_relayout"

	pack $w.label -side left -padx 1 -pady 1
	pack $w.label_ca -side left -padx 1 -pady 1 -fill x
	pack $w.inputca -side left -padx 1 -pady 1 -fill x
	pack $w.label_cr -side left -padx 1 -pady 1 -fill x
	pack $w.inputcr -side left -padx 1 -pady 1 -fill x

	pack $w.label_iter -side left -padx 1 -pady 1 -fill x
	pack $w.inputiter -side left -padx 1 -pady 1 -fill x
	pack $w.relayout -side right -padx 1 -pady 1 -fill both
#	pack $w.bar -fill x -expand 1

	if {$NETWORK_MODEL == "AutoNetworkModel"} {
		bind $w.inputca <Return> { nam_relayout }
		bind $w.inputcr <Return> { nam_relayout }
		bind $w.inputiter <Return> { nam_relayout }
	}
}

proc displaypanel {panel after value side fill expand args} {
    if {$value==1} {
	set str "pack $panel -side $side -fill $fill -expand $expand"
	if {$after!=""} {
	    set str "$str -after $after"
	}
	eval $str
    } else {
	pack forget $panel
    }
}

proc highlight {w mode} {
    global prevbutton
    if {$mode==1} {
	$prevbutton.b configure -relief raised
	$prevbutton.f configure -background [option get . background Nam]
	set prevbutton $w
    }
    $w.f configure -background seagreen
    $w.f configure -relief sunken
}

proc back_step { } {
        global running
        if $running { stop 1 }
	backFrame
	peer_cmd 1 backFrame
}

proc toggle_pause { } {
        global running
        if $running {
		stop 1
	} else {
		play 1
	}
}

proc single_step { } {
        global running
        if $running { stop 1 }
        nextFrame
	peer_cmd 1 nextFrame
}

proc dead name {
	global peers
	set i [lsearch -exact $peers $name]
	set peers [lreplace $peers $i $i]
}

proc done { } {
        peer_cmd 1 "dead \"[winfo name .]\""
#	peer_cmd 1 "destroy ."
#	tkgraph_cmd 1 "destroy ."
        destroy .
}

proc all_done { } {
	peer_cmd 1 "destroy ."
	tkgraph_cmd 1 "destroy ."
	destroy .
}

proc remote_change_rate r {
        global timeStep stepDisp
        set timeStep $r
        set stepDisp [step_format $r]
}

proc change_rate inc {
        global timeStep stepDisp
        if $inc {
	        set timeStep [expr $timeStep + $timeStep*0.05]
	} else {
	        set timeStep [expr $timeStep - $timeStep*0.05]
	}	    
        set stepDisp [step_format $timeStep]
	peer_cmd 1 "remote_change_rate $timeStep"
}

set montab(l) graph
set montab(n) ""
set montab(p) monitor
set montab(r) monitor
set montab(a) monitor

proc start_info { netView x y } {
    global running resume nowDisp tracefile montab
    if {($running)||([winfo exists $netView.f]&&($resume==1))} {
	set resume 1 
    } else {
	set resume 0
    }
    stop 1
    set text [string trim [$netView info $nowDisp $x $y] "\n"]
    if { [string length $text] > 0 } {
	set name [$netView getname $nowDisp $x $y]
	set type [lindex $name 0]
	if {$type==""} {
	    puts "got an empty object name returned from object $text!"
	    return
	}
	catch {destroy $netView.f}
	frame $netView.f -relief groove -borderwidth 2
	message $netView.f.msg -width 8c -text $text \
		-font [option get . smallfont Nam]
	pack $netView.f.msg -side top
	frame $netView.f.f
	pack $netView.f.f -side top -fill x -expand true
	if {$montab($type)=="monitor"} {
	    button $netView.f.f.b -relief raised -highlightthickness 0 \
		    -text "Monitor" \
		    -command "monitor $netView $nowDisp $x $y; end_info $netView"
	    pack $netView.f.f.b -side left -fill x -expand true
	} elseif {$montab($type)=="graph"} {
	    button $netView.f.f.b2 -relief raised -highlightthickness 0 \
		    -text "Graph" \
		    -command "graph_info $netView $x $y"
	    pack $netView.f.f.b2 -side left -fill x -expand true
	}
	button $netView.f.f.d -relief raised -highlightthickness 0 \
		-text "Dismiss" \
		-command "end_info $netView"
	pack $netView.f.f.d -side left -fill x -expand true
	set nh [winfo height $netView]
	set nw [winfo width $netView]
	#this is a hack
	place $netView.f -x $nw -y $nh
	update
	set ih [winfo reqheight $netView.f]
	set iw [winfo reqwidth $netView.f]
	set px $x
	set py $y
	if {($px+$iw)>$nw} {
	    set px [expr $x-$iw]
	    if {$px<0} {set px 0}
	}
	if {($py+$ih)>$nh} {
	    set py [expr $y-$ih]
	    if {$py<0} {set py 0}
	}
	place $netView.f -x $px -y $py
    } else {
	end_info $netView
    }
}

proc graph_info { netView x y } {
    global running resume nowDisp tracefile
    catch {destroy $netView.f}
    frame $netView.f -relief groove -borderwidth 2
    set name [$netView getname $nowDisp $x $y]
    set type [lindex $name 0]
    switch $type {
	"l" {
	    set e1 [lindex $name 1]
	    set e2 [lindex $name 2]
	    frame $netView.f.f1 -borderwidth 2 -relief groove
	    pack $netView.f.f1 -side left
	    label $netView.f.f1.l -text "Graph bandwidth" -font [smallfont]
	    pack $netView.f.f1.l -side top
	    frame $netView.f.f1.f -borderwidth 0
	    pack $netView.f.f1.f -side top
	    button $netView.f.f1.f.b2 -relief raised -highlightthickness 0 \
		    -text "Link $e1->$e2" -font [smallfont]\
		    -command "end_info $netView;\
		      viewgraph \"l $e1 $e2\" bw $tracefile"
	    pack $netView.f.f1.f.b2 -side left -fill x -expand true
	    button $netView.f.f1.f.b3 -relief raised -highlightthickness 0 \
		    -text "Link $e2->$e1" -font [smallfont] \
		    -command "end_info $netView;\
		      viewgraph \"l $e2 $e1\" bw $tracefile"
	    pack $netView.f.f1.f.b3 -side left -fill x -expand true

	    frame $netView.f.f2 -borderwidth 2 -relief groove
	    pack $netView.f.f2 -side left
	    label $netView.f.f2.l -text "Graph loss" -font [smallfont]
	    pack $netView.f.f2.l -side top
	    frame $netView.f.f2.f -borderwidth 0
	    pack $netView.f.f2.f -side top
	    button $netView.f.f2.f.b2 -relief raised -highlightthickness 0 \
		    -text "Link $e1->$e2" -font [smallfont]\
		    -command "end_info $netView;\
		      viewgraph \"l $e1 $e2\" loss $tracefile"
	    pack $netView.f.f2.f.b2 -side left -fill x -expand true
	    button $netView.f.f2.f.b3 -relief raised -highlightthickness 0 \
		    -text "Link $e2->$e1" -font [smallfont] \
		    -command "end_info $netView;\
		      viewgraph \"l $e2 $e1\" loss $tracefile"
	    pack $netView.f.f2.f.b3 -side left -fill x -expand true
	}
    }
    button $netView.f.d -relief raised -highlightthickness 0 \
	    -text "Dismiss" -font [smallfont]\
	    -command "end_info $netView"
    pack $netView.f.d -side left  -anchor sw
    set nh [winfo height $netView]
    set nw [winfo width $netView]
    #this is a hack
    place $netView.f -x $nw -y $nh
    update
    set ih [winfo reqheight $netView.f]
    set iw [winfo reqwidth $netView.f]
    set px $x
    set py $y
    if {($px+$iw)>$nw} {
	set px [expr $x-$iw]
	if {$px<0} {set px 0}
    }
    if {($py+$ih)>$nh} {
	set py [expr $y-$ih]
	if {$py<0} {set py 0}
    }
    place $netView.f -x $px -y $py
}

proc end_info {netView} { 
        global resume
	catch { destroy $netView.f }
        if $resume { play 1 }
}

proc monitor {netView now x y} {
    global windows showpanel
    set mon [$netView new_monitor $now $x $y]
    set showpanel(monitor) 1
    if {$mon>=0} {
	create_monitor $netView $mon $now
    }
}

set maxmon 0
proc create_monitor {netView mon now} {
    global windows monitors maxmon
    frame $windows(monitor).f$maxmon -borderwidth 2 -relief groove
    pack $windows(monitor).f$maxmon -side left -fill y
    set monitors($mon) $windows(monitor).f$maxmon
    set w $windows(monitor).f$maxmon
    message $w.l -aspect 300 -text "\[$mon\] [$netView monitor $now $mon]" \
	    -anchor nw -font [option get . smallfont Nam]
    pack $w.l -side top -anchor w -fill both -expand true
#    button $w.b -text "Hide" -borderwidth 1 \
#	    -command "delete_monitor $netView $mon"
#    pack $w.b -side top
    bind $w.l <Enter> \
	    "$w.l configure -background [option get . activeBackground Nam]"
    bind $w.l <Leave> \
	    "$w.l configure -background [option get . background Nam]"
    bind $w.l <1> "delete_monitor $netView $mon"
    incr maxmon
}

proc update_monitors {netView now} {
    global monitors
    foreach mon [array names monitors] {
	set text [$netView monitor $now $mon]
	if {$text==""} {
	    delete_monitor $netView $mon
	} else {
	    set text "\[$mon\] $text"
	    if {[$monitors($mon).l configure -text]!=$text} {
		$monitors($mon).l configure -text "$text"
	    }
	}
    }
}

proc delete_monitor {netView mon} {
    global monitors
    $netView delete_monitor $mon
    catch {destroy $monitors($mon)}
    unset monitors($mon)
    redrawFrame
}

proc viewgraph { object graphtype tracefile } {
    global netView now timeslider windows nam_name graphs
    puts "viewgraph $object $graphtype $tracefile"
    if {$object==""} {return}
    set graphtrace [new Trace $tracefile]
    set netgraph ""
    switch [lindex $object 0] {
	l {
	    set netgraph [new LinkNetworkGraph]
	    switch $graphtype {
		"bw" {
		    $netgraph bw [lindex $object 1] [lindex $object 2]
		}
		"loss" {
		    $netgraph loss [lindex $object 1] [lindex $object 2]
		}
	    }
	}
    }
    if {$netgraph==""} {
	return
    }
    set name [lindex $object 0]_[lindex $object 1]_[lindex $object 2]_$graphtype
    if {[winfo exists .graph.f$name]==1} {
	return
    }
    $windows(title) configure -text "Please wait - reading tracefile..."
    update
    set maxtime [$graphtrace maxtime]
    set mintime [$graphtrace mintime]
    $graphtrace connect $netgraph
    $netgraph timerange $mintime $maxtime

    #force the entire tracefile to be read
    $graphtrace settime $maxtime 1

    if {[winfo exists .graph]==0} {
	frame .graph 
	pack .graph -side top -fill x -expand true -after $timeslider(frame)
    }
    lappend graphs $netgraph
    frame .graph.f$name -borderwidth 2 -relief groove
    pack .graph.f$name -side top -expand true -fill both
    label .graph.f$name.pr -bitmap pullright -borderwidth 1 -relief raised
    bind .graph.f$name.pr <Enter> \
	    "viewgraph_label \"[viewgraph_name $object $graphtype]\" \
	       .graph.f$name .graph.f$name.pr $netgraph"
    pack .graph.f$name.pr -side left
    $netgraph view .graph.f$name.view

    #set the current time in the graph
    $netgraph settime $now

    pack .graph.f$name.view -side left -expand true \
	    -fill both
    frame .graph.f$name.l2 -width [expr $timeslider(swidth)/2] -height 30
    pack .graph.f$name.l2 -side left
    $windows(title) configure -text $nam_name
}

proc viewgraph_label {info win where netgraph} {
    if {[winfo exists $win.lbl]==0} {
	frame $win.lbl -borderwidth 2 -relief groove
	button $win.lbl.hide -text "Hide" -relief raised -borderwidth 1 \
		-highlightthickness 0 \
		-command "destroy $win;\
		rm_list_entry graphs $netgraph;\
		if {\[winfo children .graph\]=={}} {destroy .graph}"
	pack $win.lbl.hide -side left
	label $win.lbl.l -text $info -font [smallfont]
	pack $win.lbl.l -side left
    }
    catch {
	pack $win.lbl -side left -after $where -fill y
	pack forget $where
	bind $win.lbl <Leave> \
	    "pack $where -side left -before $win.lbl;pack forget $win.lbl"
    }
}

proc rm_list_entry {var value} {
    global $var
    set res ""
    set lst [set [set var]]
    foreach el $lst {
	if {[string compare $el $value]!=0} {
	    lappend res $el
	}
    }
    set [set var] $res
}

proc viewgraph_name {name graphtype} {
    set type [lindex $name 0]
    switch $type {
	"l" {
	    switch $graphtype {
		"bw" {
		    return "Bandwidth used on link [lindex $name 1]->[lindex $name 2]"
		}
		"loss" {
		    return "Packets dropped on link [lindex $name 1]->[lindex $name 2]"
		}
	    }
	}
    }
    return unknown
}

proc graph_init graphInput {
	global prevAckId prevPktId interval intervalStart intervalEnd range \
	       clearDataCmd graphName delay01 graphTool
	set prevAckId 0
	set prevPktId 0
	set delay01 0
	set clearDataCmd ""
	set intervalStart 0.0
	set intervalEnd [expr $intervalStart + $interval]
	exec tkgraph $graphInput &
	after 1000
	set interps [winfo interps]
	foreach i $interps {
		if [regexp ^tkgraph* $i] {
			after 1000
			set graphTool $i
			set graphName [send $i {graph_name}]
			tkgraph_cmd 0 \
			  "update_graph $graphName $intervalStart $intervalEnd"
			break
		}
	}
}

proc tkgraph_cmd { async cmd } {
	global graphName graphTool
	if { [string length $graphName] > 0 } {
		remote_cmd $async $graphTool $cmd
	}
}
proc remote_cmd { async interp cmd } {
	if $async {
		set rcmd "send -async \"$interp\" {$cmd}"
	} else {
		set rcmd "send \"$interp\" {$cmd}"
	}
	eval $rcmd
}

#proc master_cmd { async cmd } {
#	global master
#	if { [llength $master] > 0 } {
#		remote_cmd $async $master $cmd
#	}
#}

proc peer_cmd { async cmd } {
	global peers
	foreach s $peers {
		remote_cmd $async $s $cmd
	}
}

proc peer_init name {
	peer $name 0
	peer_cmd 0 "peer \"[winfo name .]\" 1"
}

proc peer { name remote } {
	global peers
	if { $remote } {
		peer_cmd 1 "peer \"$name\" 0"
		foreach s $peers {
			remote_cmd 1 $name "peer \"$s\" 0"
		}
	}
	lappend peers $name
}

#read events from the tracefile and parse them as layout commands until
#we meet a read event
proc nam_layout { netModel tracefile } {
    set file [open $tracefile "r"]
    set time "*"
    while {([eof $file]==0)&&([string compare $time "*"]==0)} {
	set line [gets $file]
	set cmd [lindex $line 0]
	set time [get_trace_item "-t" $line]
	if {[string compare $time "*"]!=0} {break}
	switch $cmd {
	    "n" {
		layout_node $netModel [lrange $line 1 end]
	    }
	    "l" {
		layout_link $netModel [lrange $line 1 end]
	    }
	    "L" {
		layout_lanlink $netModel [lrange $line 1 end]
	    }
	    "X" {
		layout_lan $netModel [lrange $line 1 end]
	    }
	    "q" {
		layout_queue $netModel [lrange $line 1 end]
	    }
	    "c" {
		layout_color $netModel [lrange $line 1 end]
	    }
	}
    }
    close $file
}

proc nam_prelayout { tracefile } {
    #nam_prelayout parses the tracefile before any layout is attempted
    #to see if there is enough header information to use manual layout
    #or whether auto-layout must be performed.
    global NETWORK_MODEL
    set file [open $tracefile "r"]
    set time "*"
    set NETWORK_MODEL NetworkModel
    while {([eof $file]==0)&&([string compare $time "*"]==0)} {
	set line [gets $file]
	set cmd [lindex $line 0]
	set time [get_trace_item "-t" $line]
	if {[string compare $time "*"]!=0} {break}
	switch $cmd {
	    "l" {
		set direction [get_trace_item "-o" $line]
		if {$direction==""} {
		    set NETWORK_MODEL AutoNetworkModel
		}
	    }
	}
    }
    close $file
}

proc get_trace_item { tag traceevent} {
    set next 0
    foreach item $traceevent {
	if { $next == 1 } {
	    return $item
	}
	if { $item == $tag } {
	    set next 1
	} 
    }
    return ""
}

proc layout_node {netModel line} {
    set color black
    set shape circle
    set src ""
    for {set i 0} {$i < [llength $line]} {incr i} {
	set item [string range [lindex $line $i] 1 end]
	switch $item {
	    "s" {
		set src [lindex $line [expr $i+1]]
	    }
	    "v" {
		set shape [lindex $line [expr $i+1]]
	    }
	    "c" {
		set color [lindex $line [expr $i+1]]
	    }
	}
	if {[string range $item 0 0]=="-"} {incr i}
    }
    if {$src==""} {
	puts "Error in parsing tracefile line:\n$line\n"
	exit
    }
    $netModel node $src $shape $color
    
}

proc layout_lan {netModel line} {
    set name ""
    set rate ""
    set delay ""
    set orient down
    for {set i 0} {$i < [llength $line]} {incr i} {
	set item [string range [lindex $line $i] 1 end]
	switch $item {
	    "n" {
		set name [lindex $line [expr $i+1]]
	    }
            "r" {
                set rate [lindex $line [expr $i+1]]
            }
            "D" {
                set delay [lindex $line [expr $i+1]]
            }
            "o" {
                set orient [lindex $line [expr $i+1]]
            }
	}
	if {[string range $item 0 0]=="-"} {incr i}
    }
    if {$name==""} {
	puts "Error in parsing tracefile line:\n$line\n"
	exit
    }
    set th [nam_angle $orient]
    $netModel lan $name [bw2real $rate] [time2real $delay] [expr $th]
    
}

proc layout_link {netModel line} {
    global delay01
    #orient can default because pre-layout will have kicked us into 
    #autolayout mode by this point
    set orient right
    set src ""
    set dst ""
    set rate ""
    set delay ""
    set color ""
    for {set i 0} {$i < [llength $line]} {incr i} {
	set item [string range [lindex $line $i] 1 end]
	switch "$item" {
	    "s" {
		set src [lindex $line [expr $i+1]]
	    }
	    "d" {
		set dst [lindex $line [expr $i+1]]
	    }
	    "r" {
		set rate [lindex $line [expr $i+1]]
	    }
	    "D" {
		set delay [lindex $line [expr $i+1]]
	    }
	    "o" {
		set orient [lindex $line [expr $i+1]]
	    }
	    "c" {
		set color [lindex $line [expr $i+1]]
	    }
	}
	if {[string range $item 0 0]=="-"} {incr i}
    }
    if {($src=="")||($dst=="")||($rate=="")||($delay=="")} {
	puts "Error in parsing tracefile line:\n$line\n"
	exit
    }
    set th [nam_angle $orient]
    $netModel link $dst $src \
	    [bw2real $rate] [time2real $delay] [expr $th + 1]
    set result [$netModel link $src $dst \
	    [bw2real $rate]  [time2real $delay] $th]
    if { $src == 0 && $dst == 1 } {
		set delay01 $result
    }
    
    if { $color!=""} {
	$netModel ecolor $src $dst $color
	$netModel ecolor $dst $src $color
    }
}

proc layout_lanlink {netModel line} {
    global delay01
    puts "layout_lanlink $line"
    #orient can default because pre-layout will have kicked us into 
    #autolayout mode by this point
    set orient right
    set src ""
    set dst ""
    for {set i 0} {$i < [llength $line]} {incr i} {
	set item [string range [lindex $line $i] 1 end]
	switch "$item" {
	    "s" {
		set src [lindex $line [expr $i+1]]
	    }
	    "d" {
		set dst [lindex $line [expr $i+1]]
	    }
	    "o" {
		set orient [lindex $line [expr $i+1]]
	    }
	}
	if {[string range $item 0 0]=="-"} {incr i}
    }
    if {($src=="")||($dst=="")} {
	puts "Error in parsing tracefile line:\n$line\n"
	exit
    }
    set th [nam_angle $orient]
    puts "lanlink $dst $src $th"
    $netModel lanlink $dst $src $th
}

proc layout_queue {netModel line} {
    for {set i 0} {$i < [llength $line]} {incr i} {
	set item [string range [lindex $line $i] 1 end]
	switch $item {
	    "s" {
		set src [lindex $line [expr $i+1]]
	    }
	    "d" {
		set dst [lindex $line [expr $i+1]]
	    }
	    "a" {
		set angle [lindex $line [expr $i+1]]
	    }
	}
	if {[string range $item 0 0]=="-"} {incr i}
    }
    $netModel queue $src $dst $angle
}

proc layout_color {netModel line} {
    for {set i 0} {$i < [llength $line]} {incr i} {
	set item [string range [lindex $line $i] 1 end]
	switch $item {
	    "i" {
		set ix [lindex $line [expr $i+1]]
	    }
	    "n" {
		set name [lindex $line [expr $i+1]]
	    }
	}
	if {[string range $item 0 0]=="-"} {incr i}
    }
    $netModel color $ix $name
}

#
# do more iterations to layout network model 
#
proc nam_relayout {} {
	global netModel ITERATIONS KCa KCr now netView
	$netModel instvar INCR_ITERATIONS_ KCa_ KCr_
	set KCa_ $KCa
	set KCr_ $KCr
	set INCR_ITERATIONS_ $ITERATIONS
	$netModel relayout
	settime $now
	update idletasks
}

# nam_init trace-name [g=graph-input] [i=graph-interval]
# where trace-name is the nam trace input file
#       graph-input is the input tcl file to tkgraph (optional)
#       graph-interval is the graph interval to be used (optional)
#                      and is only meaningful when a graph input file
#                      is provided
proc nam_init { trace_file args } {
	global windows
	. configure -background [option get . background Nam]

	global trace now mintime range maxtime prevTime prevRate \
               rateSlider netModel currRate graphName \
	       interval running peers peerName granularity direction \
	       NETWORK_MODEL ITERATIONS KCa KCr magnification \
	       annoBox LastAnnoTime showpanel tracefile nam_name graphs \
               netViews
        AutoNetworkModel instvar KC_ INCR_ITERATIONS_ KCa_ KCr_
        set nam_name "  VINT Network Animator v[version]"
        set tracefile $trace_file
	set KCa $KCa_		;# to interface with TK's entry
	set KCr $KCr_
	set ITERATIONS $INCR_ITERATIONS_
	set direction 1
	set LastAnnoTime 0	;# annotation time monotonically increasing so 
				;# they'll be added only once
        set graphs ""
        nam_prelayout $tracefile
	set netModel [new $NETWORK_MODEL]
	set trace [new Trace $tracefile]
	set now [$trace mintime]
	set mintime $now
        set maxtime [expr [$trace maxtime] + .05]
	set range [expr $maxtime - $mintime]
        set prevTime $mintime
	$trace connect $netModel

	frame .view


#	nam_config $netModel
        nam_layout $netModel $tracefile
	$netModel layout
        frame .view.f
        #frame is just to sink the netview
        frame .view.f.f -borderwidth 2 -relief sunken
	$netModel view .view.f.f.net
        set netView .view.f.f.net
	set netViews $netView
        pack .view.f.f.net -side top -expand true -fill both

        #build the menu bar
        frame .menu -borderwidth 0
	build.menus .menu
        pack .menu -side top -fill x

        #build the control panel with rew, ffwd, play, etc
	frame .ctrl0 -relief flat -borderwidth 0
	build.p1 .ctrl0 $netView
        pack .ctrl0 -side top -fill x

	set running 0
#        scrollbar .view.f.hsb -orient horizontal -width 10 -borderwidth 1 \
#		-command "viewPan $netView xview"
        $netView xscroll .view.f.hsb
        scrollbar .view.f.hsb -orient horizontal -width 10 -borderwidth 1 \
		-command "$netView xview"
        .view.f.hsb set 0.0 1.0
        pack .view.f.hsb -side bottom -fill x
        pack .view.f.f -side top -fill both -expand true

        frame .view.f2
#        scrollbar .view.f2.vsb -orient vertical -width 10 -borderwidth 1 \
#		-command "viewPan $netView yview"
        $netView yscroll .view.f2.vsb
        scrollbar .view.f2.vsb -orient vertical -width 10 -borderwidth 1 \
		-command "$netView yview"
        .view.f2.vsb set 0.0 1.0
        pack .view.f2.vsb -side top -fill y -expand true
        frame .view.f2.l -width 12 -height 12
        pack .view.f2.l -side top

	set interval [expr $range / 7.]
	set graphName ""
	set graphInput ""
	set peerName ""
	set peers ""
	set i 0
	foreach a $args {
		set x [lindex $args [expr $i+1]]
		set aa [split $a "="]
		set pn [format "%s %s" [lindex $aa 1] $x]
		switch [lindex $aa 0] {
			g {set graphInput [lindex $aa 1]}
			i {set interval [lindex $aa 1]}
			p {
				if {[string length $x] > 0} {
					set peerName $pn
				} else {
					set peerName [lindex $aa 1]
				}
			}
		}
		incr i
	}

	if { [llength $graphInput] > 0 } {
		graph_init $graphInput
	}
	if { [llength $peerName] > 0 } {
		peer_init $peerName
	}
	frame .view.ctrl -borderwidth 2 -relief groove
	build.v0 $netView .view.ctrl
	pack .view.ctrl -side left -fill y
        pack .view.f2 -side right -fill y
        pack .view.f -side left -fill both -expand true
	
	frame .monwin -relief groove -borderwidth 2
	button .monwin.l -bitmap monitors -borderwidth 2 -relief groove \
		-command "set showpanel(monitor) 0"
	pack .monwin.l -side left -fill y
	set windows(monitor) .monwin
	trace variable showpanel(monitor) w \
		{displaypanel .monwin .view $showpanel(monitor) \
		top x false}
	set showpanel(monitor) 0
	
	frame .ctrl -relief flat -borderwidth 0
#	frame .ctrl.p0 -relief flat -borderwidth 0
#	build.p0 .ctrl.p0
	frame .ctrl.p0b -relief flat -borderwidth 0
	build.p0b .ctrl.p0b

	frame .ctrl.tl -relief flat -borderwidth 0
#	build.tl .ctrl.tl
	build.al .ctrl.tl
	set showpanel(annotate) 1

#	global sim_annotation
#	set sim_annotation ""

	if {$NETWORK_MODEL == "AutoNetworkModel"} {
	        set showpanel(autolayout) 1
	        trace variable showpanel(autolayout) w \
		    {displaypanel .ctrl.p2 .ctrl.p0b $showpanel(autolayout)\
		     top x false}
		frame .ctrl.p2 -relief flat -borderwidth 0
		build.p2 .ctrl.p2
		pack .ctrl.p0b .ctrl.p2 .ctrl.tl  \
			-side top -fill x
	} else {
	        set showpanel(autolayout) 0
		pack .ctrl.p0b .ctrl.tl -side top -fill x
	}

	set windows(control) .ctrl

	pack .ctrl -side bottom -fill x
	pack .view -side top -fill both -expand true

	wm minsize . 200 200

	settime $now
	set_rate $currRate 1

        window_bind .
        view_bind $netView
}

set viewctr 0
proc new_view {} {
    global netModel viewctr nam_name netViews
    toplevel .v$viewctr
    set w .v$viewctr
    incr viewctr
    wm title $w $nam_name
    frame $w.f
    #frame is just to sink the newView
    frame $w.f.f -borderwidth 2 -relief sunken
    $netModel view $w.f.f.net
    set newView $w.f.f.net
    lappend netViews $newView
    pack $w.f.f.net -side top -expand true -fill both
    
    $newView xscroll $w.f.hsb
    scrollbar $w.f.hsb -orient horizontal -width 10 -borderwidth 1 \
	    -command "$newView xview"
    $w.f.hsb set 0.0 1.0
    pack $w.f.hsb -side bottom -fill x
    pack $w.f.f -side top -fill both -expand true
    
    frame $w.f2
    $newView yscroll $w.f2.vsb
    scrollbar $w.f2.vsb -orient vertical -width 10 -borderwidth 1 \
	    -command "$newView yview"
    $w.f2.vsb set 0.0 1.0
    pack $w.f2.vsb -side top -fill y -expand true
    frame $w.f2.l -width 12 -height 12
    pack $w.f2.l -side top
    frame $w.ctrl -borderwidth 2 -relief groove
    build.v0 $newView $w.ctrl
    pack $w.ctrl -side left -fill y
    pack $w.f2 -side right -fill y
    pack $w.f -side left -fill both -expand true
    window_bind $w
    view_bind $newView
}

proc window_bind {w} {
        bind $w <q> { done }
	bind $w <Q> { all_done }
	bind $w <Control-c> { done }

        bind $w <b> { back_step }
        bind $w <B> { back_step }
	
	bind $w <c> { play 1 }
	bind $w <C> { play 1 }
	bind $w <f> { fast_fwd }
	bind $w <F> { fast_fwd }
	bind $w <n> { next_event }
	bind $w <N> { next_event }
	bind $w <p> { stop 1 }
	bind $w <P> { stop 1 }
	bind $w <r> { rewind }
	bind $w <R> { rewind }
	bind $w <u> { time_undo }
	bind $w <U> { time_undo }
	bind $w <x> { rate_undo }
	bind $w <X> { rate_undo }
	bind $w <period> { change_rate 1 }
	bind $w <greater> { change_rate 1 }
	bind $w <comma> { change_rate 0 }
	bind $w <less> { change_rate 0 }
}

proc view_bind {netView} {
	# We need these keys in input boxes, so only bind them to .view
	bind $netView <space> { toggle_pause }
	bind $netView <Control-d> { done }
	bind $netView <Return> { single_step }
	bind $netView <BackSpace> { back_step }
	bind $netView <Delete> { back_step }
	bind $netView <0> { reset }

	bind $netView <ButtonPress> "start_info $netView \%x \%y"
#	bind $netView <ButtonRelease-3> "end_info $netView"

}



#
# helper functions
#
proc nam_angle { v } {
	switch -regexp $v {
	       ^[0-9\.]+deg$    { regexp  {([0-9\.]+)deg$} $v a dval
     			          return [expr {fmod($dval,360)/180}] }

		^up-right$ -
		^right-up$	{ return 0.25 }


		^up$		{ return 0.5 }

		^up-left$ -
		^left-up$	{ return 0.75 }

		^left$		{ return 1. }

		^left-down$ -
		^down-left$	{ return 1.25 }

		^down$		{ return 1.5 }

		^down-right$ -
		^right-down$	{ return 1.75 }

		default		{ return 0.0 }
	}
}

proc ncolor {n0 color} {
	global netModel
	$netModel ncolor $n0 $color
}

proc ecolor {n0 n1 color} {
	global netModel
	$netModel ecolor $n0 $n1 $color
	$netModel ecolor $n1 $n0 $color
}

global env
if [info exists env(HOME)] {
	set initFile $env(HOME)/.nam.tcl
	if [file exists $initFile] {
		source $initFile
	}
}

# we'll build a new annotation listbox for this
proc build.al { w } {
	global annoBox showpanel
        frame $w.spaceral -borderwidth 0 -highlightthickness 0 \
		-height 0 -width 10
        pack $w.spaceral -side top -padx 0 -pady 0
        frame $w.f -borderwidth 0 -highlightthickness 0
	frame $w.f.f

	listbox $w.f.f.a -xscrollcommand "$w.f.f.ah set" -yscrollcommand "$w.f.f2.av set" -height 3
	pack $w.f.f.a -fill both -side top -expand true 
	set annoBox $w.f.f.a

	scrollbar $w.f.f.ah -orient horizontal -width 10 -borderwidth 1 \
		-command "$w.f.f.a xview"
	$w.f.f.ah set 0.0 1.0
	pack $w.f.f.ah -side bottom -fill x

	frame $w.f.f2
	pack $w.f.f2 -side left -fill y
	scrollbar $w.f.f2.av -orient vertical -width 10 -borderwidth 1 \
			-command "$w.f.f.a yview"
	$w.f.f2.av set 0.0 1.0
	pack $w.f.f2.av -side top -fill y -expand true
        pack $w.f.f -side left -fill both -expand true
        trace variable showpanel(annotate) w \
		"displaypanel $w.f $w.spaceral \[set showpanel(annotate)\] \
		    top both true"

	bind $w.f.f.a <Double-ButtonPress-1> " jump_to_annotation $w.f.f.a"
}

# obsolete and should be removed
proc build.tl { w } {
	set f [smallfont]
	frame $w.bar -relief ridge -borderwidth 2
#	label $w.bar.title -text "Annot.:" -font $f -foreground Blue
	label $w.bar.l -textvariable sim_annotation -font $f -foreground Blue
#	pack $w.bar.title -side left -padx 2
	pack $w.bar.l -side left -fill x
	pack $w.bar -side top -expand 1 -fill x
}

# Can't use 'now'. Observed when sim_annotation gets executed, 'now' may not 
# be exactly the desired time.
proc sim_annotation { time args } {
	global annoBox annoTimeList LastAnnoTime

	# only add annotation once and never delete them
	if {$time > $LastAnnoTime} {
		# insert when going forward
		$annoBox insert end "$args"
		set size [$annoBox size]
		set annoTimeList([expr $size - 1]) $time
		set LastAnnoTime $time
		puts "At time $time, added $args"
	}
}

proc jump_to_annotation { anno } {
	global annoTimeList
	set idx [$anno index active]
	puts "Jumping to time $annoTimeList($idx)"
	settime $annoTimeList($idx)
}


#WWW "browser" - setup big table of what affects what
set stackthese "h1 h2 h3 h4 h5 h6 b i ul ol dl a pre"
set www_cancels(/h1) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(/h2) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(/h3) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(/h4) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(/h5) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(/h6) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(h1) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(h2) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(h3) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(h4) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(h5) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(h6) "h1 h2 h3 h4 h5 h6 i b"
set www_cancels(/b) "b"
set www_cancels(/strong) "strong"
set www_cancels(/i) "i"
set www_cancels(/ul) "ul"
set www_cancels(/ol) "ol"
set www_cancels(/dl) "dl"
set www_cancels(/a) "a"
set www_cancels(/pre) "pre"
set www_sets(h1) {{bold 1} {size 18}}
set www_sets(h2) {{bold 1} {size 16}}
set www_sets(h3) {{bold 1} {size 14}}
set www_sets(h4) {{bold 1} {size 12}}
set www_sets(h5) {{bold 1} {size 10}}
set www_sets(h6) {{bold 1} {size 10}}
set www_sets(i) {{italic 1} {bold 0}}
set www_sets(b) {{italic 0} {bold 1}}
set www_sets(strong) {{italic 0} {bold 1}}
set www_sets(verbatim) {{verbatim 1} {font courier}}
set www_sets(normal) {{bold 0} {italic 0} {font helvetica} {size 12} {nl 0} {hr 0} {ltype ""} {bullet 0} {verbatim 0} {indent 0} {href ""}}
set www_sets(ul) {{ltype ul} {indent [expr $indent+4]}}
set www_sets(ol) {{ltype ol} {indent [expr $indent+4]}}
set www_sets(dl) {{ltype dl} {indent [expr $indent+4]}}
set www_sets(a) {{href XXX}}
set www_runs(a) {extract_anchor}
set www_runs(img) {extract_image}

#www_sets is for tags that get stacked and need to be set continuously
#www_fsets is only run the first time 
set www_fsets(p) {{nl 2}}
set www_fsets(br) {{nl 1}}
set www_fsets(hr) {{hr 1}}
set www_fsets(h1) {{nl 2}}
set www_fsets(h2) {{nl 2}}
set www_fsets(h3) {{nl 2}}
set www_fsets(h4) {{nl 2}}
set www_fsets(h5) {{nl 2}}
set www_fsets(h6) {{nl 2}}
set www_fsets(ul) {{nl 0}}
set www_fsets(ol) {{nl 0}}
set www_fsets(dl) {{nl 0}}
set www_fsets(/h1) {{nl 2}}
set www_fsets(/h2) {{nl 2}}
set www_fsets(/h3) {{nl 2}}
set www_fsets(/h4) {{nl 2}}
set www_fsets(/h5) {{nl 2}}
set www_fsets(/h6) {{nl 2}}
set www_fsets(/ul) {{nl 1}}
set www_fsets(/ol) {{nl 1}}
set www_fsets(/dl) {{nl 1}}
set www_fsets(li) {{nl 1} {bullet 1}}
set www_fsets(dt) {{nl 1}}

set fonts(normal) -*-helvetica-medium-r-normal--12-*
set fonts(helvetica,0,0,16) -*-helvetica-medium-r-normal--17-*
set fonts(helvetica,1,0,16) -*-helvetica-bold-r-normal--17-*
set fonts(helvetica,1,1,16) -*-helvetica-bold-o-normal--17-*
set fonts(helvetica,1,0,18) -*-helvetica-bold-r-normal--18-*
set fonts(helvetica,1,1,18) -*-helvetica-bold-o-normal--18-*
set fonts(helvetica,1,0,14) -*-helvetica-bold-r-normal--14-*
set fonts(helvetica,1,1,14) -*-helvetica-bold-o-normal--14-*
set fonts(helvetica,1,0,12) -*-helvetica-bold-r-normal--12-*
set fonts(helvetica,1,1,12) -*-helvetica-bold-o-normal--12-*
set fonts(helvetica,0,1,12) -*-helvetica-medium-o-normal--12-*
set fonts(helvetica,1,0,10) -*-helvetica-bold-r-normal--10-*

proc parse_html {w entry text } {
    global www_sets www_fsets www_cancels www_runs stackthese fonts href href_keep www_win
    set www_win $w
    set stack normal
    set size normal
    set font helvetica
    set bold 0
    set italic 0
    set underline 0
    set verbatim 0
    set indent 0
    set nl 0
    set hr 0
    set prevnl 0
    set href ""
    set bullet 0
    set ltype ""
    set parts [split $text "<>"]
    set istag 0
    $w delete 1.0 end
    foreach part $parts {
	if {$istag==1} {
	    set tag [string tolower [lindex $part 0]]
	    set c ""
	    catch {set c $www_cancels($tag)}
	    set rm 0
	    set newstack {}
	    foreach frame $stack {
		set newframe {}
		if {([lsearch -exact $c $frame]>=0)&&($rm==0)} {
		    set rm 1
		} else {
		    lappend newstack $frame
		}
	    }
	    set stack $newstack
	    set newframe $tag
#	    puts "$tag, $stack"

	    #set all the stuff corresponding to this stack
	    foreach frame "$stack $newframe" {
		set t [lindex $frame 0]
		set s ""
		catch {set s $www_sets($t)}
		foreach item $s {
		    set cmd "set $item"
#		    puts "    $cmd"
		    eval $cmd
		}
	    }

	    #now deal with the one off stuff for this tag
	    set s ""
	    catch {set s $www_fsets($tag)}
	    foreach item $s {
		set cmd "set $item"
#		puts "    $cmd"
		eval $cmd
	    }

	    #now run any specific code for this tag
	    set s ""
	    catch {set s $www_runs($tag)}
	    foreach item $s {
		set cmd "$item $part"
#		puts "    $cmd"
		eval $cmd
	    }

	    if {[lsearch -exact $stackthese $tag]>=0} {
		lappend stack $newframe
	    }

	} else {
	    if {$hr==1} {
		$w insert end "\n_____________________________________________________\n"
		if {$prevnl==0} {set prevnl 1}
	    }

	    #handle all the newlines
	    for {set i 0} {$i < [expr $nl-$prevnl]} {incr i} {
		$w insert end "\n"
	    }
	    set prevnl $nl

	    #sort out the indentation
	    if {$nl>0} {
		for {set i 0} {$i < $indent} {incr i} {
		    $w insert end " "
		}
	    }

	    #add any bullets
	    if {$bullet!=0} {
		$w insert end "* "
	    }

	    set ix [$w index end-1c]
	    if {$nl>0} {
		set part [string trimleft $part]
	    }
	    if {$verbatim==0} {
		set part [remove_nl $part]
		if {[string length $part]>0} {
		    set prevnl 0
		}
		$w insert end $part
	    } else {
		$w insert end $part
	    }

	    if {($size!=12)||($bold==1)||($italic==1)||($href!="")||\
		    ($font!="helvetica")} {
		global tagnum
		incr tagnum
		set ix2 [$w index end-1c]
		$w tag add t$tagnum $ix end-1c
	    }
	    if {($size!=12)||($bold==1)||($italic==1)||\
		    ($font!="helvetica")} {
		set thisfont $fonts(normal)
		catch {
		    set thisfont $fonts($font,$bold,$italic,$size)
		}
		$w tag configure t$tagnum -font $thisfont
	    }
	    if {$href!=""} {
		$w tag configure t$tagnum -foreground red
		$w tag bind t$tagnum <Enter> "set_entry $entry $href_keep"
		$w tag bind t$tagnum <Leave> "set_entry $entry {}"
		$w tag bind t$tagnum <1> "goto $href_keep $w $entry"
	    }
	    
	}
	set istag [expr 1-$istag]
    }
}

proc set_entry {entry str} {
    $entry delete 0 end
    $entry insert 0 $str
}

proc extract_anchor args {
    global href_keep href
    set href_keep ""
    set href ""
    foreach arg $args {
	if {[string range [string tolower $arg] 0 4]=="href="} {
	    set href_keep [string trim [string range $arg 5 end] {"}]
	    set href XXX
	}	    
    }
}

set wix 0
proc extract_image args {
    global www_win wix
    set img ""
    foreach arg $args {
	if {[string range [string tolower $arg] 0 3]=="src="} {
	    set img [string trim [string range $arg 4 end] {"}]
	}	    
    }
    $www_win window create end -create "label $www_win.w$wix -bitmap $img"
    incr wix
}

proc goto {url win entry} {
    global help_text
    set res "xxxx"
    set scheme [lindex [split $url ":"] 0]
    if {$scheme=="help"} {
	parse_html $win $entry $help_text([string range $url 5 end])
	return
    }
    set proggy netscape
    catch {
        set res [exec $proggy -display :0 -remote openURL($url)]
    }
    if {$res=="xxxx"} {
        catch {
            set res [exec $proggy -display :0.1 -remote openURL($url)]
        }
    }
    if {$res=="xxxx"} {
        puts "failed to pass URL to $proggy"
    }
}

set tagnum 0

proc lreverse {list} {
    set res {}
    foreach el $list {
	set res [concat [list $el] $res]
    }
    return $res
}

proc remove_nl {str} {
    set parts [split $str "\n\r"] 
    set res ""
    set space 1
    for {set i 0} {$i<[llength $parts]} {incr i} {
	set part [lindex $parts $i]
	if {$i>0} {
	    set part [string trimleft $part]
	} else {
	    if {$part==""} {set space 0}
	}
	if {($i+1)<[llength $parts]} {
	    set part [string trimright $part]
	}
	if {$space==0} {
	    set res "$res "
	    set space 1
	} 
	if {[string length $part] > 0} {
	    set res "$res$part"
	    set space 0
	}
    }
    return $res
}


set help_text(about) "<h1>VINT/UCB Network Animator (nam)</h1>
This version of nam is highly experimental - there will be bugs!.  
Please mail <I><a href=mailto:ns-developers@mash.cs.berkeley.edu>ns-developers@mash.cs.berkeley.edu</a></I> if you 
encounter any bugs, or with suggestions for desired functionality.
<H2>History</h2>
The network animator ``nam''
began in 1990 as a simple tool for animating packet trace data.
This trace data is typically
derived as output from a network simulator like
<a href=http://www-mash.cs.berkeley.edu/ns/>ns</a>
or from real network measurements, e.g., using
<a href=ftp://ftp.ee.lbl.gov/tcpdump.tar.Z>tcpdump</a>.
<a href=http://www.cs.berkeley.edu/~mccanne>Steven McCanne</a>
wrote the original version as a member of the
<a href=http://www-nrg.ee.lbl.gov/>Network Research Group</a> at the
<a href=http://www.lbl.gov/>Lawrence Berkeley National Laboratory</a>,
and occasionally improved the design as he needed it in
his research.  Marylou Orayani improved it further and used it
for her Master's research over summer 1995 and into spring 1996.
The nam development effort is now an ongoing
collaboration with the
<a href=http://netweb.usc.edu/vint/>VINT project</a>.
Current nam developers include
<a href=http://north.east.isi.edu/~mjh/>Mark Handley</a>, 
<a href=http://www.isi.edu/~haoboy/>Haobo Yu</a> and <a href=http://http://www.isi.edu/~johnh/>John Heidemann</a>.
<P>
For information about ns and nam, please see
<i><a href=http://mash-www.cs.berkeley.edu/ns/nam.html>http://mash-www.cs.berkeley.edu/ns/nam.html</a></i>.
<h2>Funding</h2>
Nam is currently funded by DARPA through the VINT project at LBL under
DARPA grant DABT63-96-C-0105, at USC/ISI under DARPA grant
ABT63-96-C-0054, at Xerox PARC under DARPA grant DABT63-96-C-0105. 
<H2>Copyright</H2>
Nam contains source code with the following copyrights:
<p>
Copyright (c) 1991-1994 Regents of the University of California.
<br>
Copyright (c) 1997 University of Southern California
<P>
<hr>
<i><a href=help:help>Help Index</a><I>
"

set help_text(help) "
<H1>Nam Quick Help</H1>
<B>Use the controls to move the animation through time:</B>
<DL>
<DT><img src=rew>Rewind the animation by 0.05 seconds
<DT><img src=back>Play the animation backwards
<DT><img src=stop>Stop the playing of the animation
<DT><img src=play>Play the animation normally
<DT><img src=ff>Fast forward the animation by 0.05 seconds
<DT><img src=eject>Exit nam
</DL>
<P>
When the animation is running
<UL><LI>Click the left button on an object in the display window to view information about that object.
<LI>Select \"monitor\" from the resulting popup window to <a href=help:monitors>monitor that object</a> over time.
<LI>The \"step\" slider can be used to control how fast time flows in the animation.
The \"time\" slider can be used to move to a specific point in time.
</UL>
<P>
<hr>
<i><a href=help:about>About Nam</a><I>
"

set help_text(monitors) "
<H1>Nam Help: Monitors</H1>
You can set monitors on various animation objects to examine 
information about them over time.
<H2>Packet Monitors</H2>
To initiate a packet monitor, click the left button on the packet 
you wish to examine.  Information that nam has about that packet 
will then be displayed, and you are given the option to set a 
monitor on the packet.
<P>
Setting a monitor on a packet labels the packet so that you can 
more easily watch its flow through the network and through queues.
An associated panel appears in the monitors window listing information 
about the packet if the packet is visible.  
<P>
Click on the monitor panel for a packet to remove that monitor.
<H2>Agent Monitors</H2>
Protocol agents are displayed alongside the node they are 
instantiated in.  To monitor an agent, click on it in the 
network display, and select \"monitor\".
Information about the agent will be displayed in the monitors 
panel, and will be updated as protocol state in the agent 
changes over time.
<P>
Click on the monitor panel for the agent to remove that monitor.
<hr>
<I><a href=help:help>Return to help index</a>
"


proc new_web {which} {
    global help_text
    if {[winfo exists .help]} {
	parse_html .help.f.f.w .help.f.ctl.e $help_text($which)
	return
    }
    toplevel .help
    wm title  .help "About nam"
    frame .help.f -relief groove -borderwidth 2
    pack  .help.f -side top -fill both -expand true
    frame .help.f.f
    pack  .help.f.f -side top -fill both -expand true
    text .help.f.f.w -width 60 -height 40 -wrap word -yscroll ".help.f.f.sb set"
    bind .help.f.f.w <1> break
    pack .help.f.f.w -side left -fill both -expand true
    scrollbar .help.f.f.sb -command ".help.f.f.w yview"
    pack .help.f.f.sb -side right -fill y
    frame .help.f.ctl -borderwidth 2 -relief groove
    pack .help.f.ctl -side top -fill x
    label .help.f.ctl.l -text "go to:"
    pack .help.f.ctl.l -side left
    entry .help.f.ctl.e -width 20 -borderwidth 1 -relief sunken
    pack .help.f.ctl.e -side left -fill x -expand true
    button .help.f.ctl.b -borderwidth 1 -relief raised -text Dismiss \
	    -command {destroy .help}
    pack .help.f.ctl.b -side left
    parse_html .help.f.f.w .help.f.ctl.e $help_text($which)
}
